﻿using System.Collections.ObjectModel;

namespace SelfReferenceHierarchyTreeViewWPF
{
	public class DataItemCollection<T> : ObservableCollection<T>
		where T : DataItem
	{
		protected override void InsertItem(int index, T item)
		{
			this.AdoptItem(item);
			base.InsertItem(index, item);
		}

		protected override void RemoveItem(int index)
		{
			this.DiscardItem(this[index]);
			base.RemoveItem(index);
		}

		protected override void SetItem(int index, T item)
		{
			this.AdoptItem(item);
			base.SetItem(index, item);
		}

		protected override void ClearItems()
		{
			foreach (T item in this)
			{
				this.DiscardItem(item);
			}
			base.ClearItems();
		}

		private void AdoptItem(T item)
		{
			item.SetOwner(this);
		}

		private void DiscardItem(T item)
		{
			item.SetOwner(null);
		}
	}
}