﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections.ObjectModel;

namespace SelfReferenceHierarchyTreeViewWPF
{
	public class ParentAwareCollection<T> : ObservableCollection<T>
		where T : DataItem
	{
		private int parentIndex;

		public ParentAwareCollection(int parentIndex)
		{
			this.parentIndex = parentIndex;
		}

		protected override void InsertItem(int index, T item)
		{
			this.AdoptItem(item);
			base.InsertItem(index, item);
		}

		protected override void RemoveItem(int index)
		{
			this.DiscardItem(this[index]);
			base.RemoveItem(index);
		}

		protected override void SetItem(int index, T item)
		{
			this.AdoptItem(item);
			base.SetItem(index, item);
		}

		protected override void ClearItems()
		{
			foreach (T item in this)
			{
				this.DiscardItem(item);
			}
			base.ClearItems();
		}

		private void AdoptItem(T item)
		{
			item.ParentID = this.parentIndex;
		}

		private void DiscardItem(T item)
		{
			item.ParentID = 0;
		}
	}
}
